from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.translation_usage_example import TranslationUsageExample


T = TypeVar("T", bound="WordTranslation")


@_attrs_define
class WordTranslation:
    """
    Attributes:
        translation (str):
        usage_examples (List['TranslationUsageExample']):
    """

    translation: str
    usage_examples: List["TranslationUsageExample"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        translation = self.translation

        usage_examples = []
        for usage_examples_item_data in self.usage_examples:
            usage_examples_item = usage_examples_item_data.to_dict()
            usage_examples.append(usage_examples_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "translation": translation,
                "usage_examples": usage_examples,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.translation_usage_example import TranslationUsageExample

        d = src_dict.copy()
        translation = d.pop("translation")

        usage_examples = []
        _usage_examples = d.pop("usage_examples")
        for usage_examples_item_data in _usage_examples:
            usage_examples_item = TranslationUsageExample.from_dict(usage_examples_item_data)

            usage_examples.append(usage_examples_item)

        word_translation = cls(
            translation=translation,
            usage_examples=usage_examples,
        )

        word_translation.additional_properties = d
        return word_translation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .body_login_for_access_token_users_token_post import BodyLoginForAccessTokenUsersTokenPost
from .conversation import Conversation
from .grammar_error import GrammarError
from .grammar_error_v2 import GrammarErrorV2
from .grammar_errors import GrammarErrors
from .grammar_errors_v2 import GrammarErrorsV2
from .http_validation_error import HTTPValidationError
from .message import Message
from .placement_test_answer import PlacementTestAnswer
from .placement_test_follow_up_question import PlacementTestFollowUpQuestion
from .placement_test_question import PlacementTestQuestion
from .placement_test_question_type import PlacementTestQuestionType
from .placement_test_questions import PlacementTestQuestions
from .send_message_chat_send_message_post_response_send_message_chat_send_message_post import (
    SendMessageChatSendMessagePostResponseSendMessageChatSendMessagePost,
)
from .token import Token
from .translation_usage_example import TranslationUsageExample
from .user import User
from .validation_error import ValidationError
from .word_translation import WordTranslation

__all__ = (
    "BodyLoginForAccessTokenUsersTokenPost",
    "Conversation",
    "GrammarError",
    "GrammarErrors",
    "GrammarErrorsV2",
    "GrammarErrorV2",
    "HTTPValidationError",
    "Message",
    "PlacementTestAnswer",
    "PlacementTestFollowUpQuestion",
    "PlacementTestQuestion",
    "PlacementTestQuestions",
    "PlacementTestQuestionType",
    "SendMessageChatSendMessagePostResponseSendMessageChatSendMessagePost",
    "Token",
    "TranslationUsageExample",
    "User",
    "ValidationError",
    "WordTranslation",
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.placement_test_answer import PlacementTestAnswer


T = TypeVar("T", bound="PlacementTestFollowUpQuestion")


@_attrs_define
class PlacementTestFollowUpQuestion:
    """
    Attributes:
        question (str):
        answers (List['PlacementTestAnswer']):
    """

    question: str
    answers: List["PlacementTestAnswer"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        question = self.question

        answers = []
        for answers_item_data in self.answers:
            answers_item = answers_item_data.to_dict()
            answers.append(answers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "question": question,
                "answers": answers,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.placement_test_answer import PlacementTestAnswer

        d = src_dict.copy()
        question = d.pop("question")

        answers = []
        _answers = d.pop("answers")
        for answers_item_data in _answers:
            answers_item = PlacementTestAnswer.from_dict(answers_item_data)

            answers.append(answers_item)

        placement_test_follow_up_question = cls(
            question=question,
            answers=answers,
        )

        placement_test_follow_up_question.additional_properties = d
        return placement_test_follow_up_question

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.placement_test_question_type import PlacementTestQuestionType

if TYPE_CHECKING:
    from ..models.placement_test_answer import PlacementTestAnswer
    from ..models.placement_test_follow_up_question import PlacementTestFollowUpQuestion


T = TypeVar("T", bound="PlacementTestQuestion")


@_attrs_define
class PlacementTestQuestion:
    """
    Attributes:
        cefr_level (str):
        question_type (PlacementTestQuestionType):
        question (str):
        answers (List['PlacementTestAnswer']):
        follow_up_questions (Union[List['PlacementTestFollowUpQuestion'], None]):
    """

    cefr_level: str
    question_type: PlacementTestQuestionType
    question: str
    answers: List["PlacementTestAnswer"]
    follow_up_questions: Union[List["PlacementTestFollowUpQuestion"], None]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        cefr_level = self.cefr_level

        question_type = self.question_type.value

        question = self.question

        answers = []
        for answers_item_data in self.answers:
            answers_item = answers_item_data.to_dict()
            answers.append(answers_item)

        follow_up_questions: Union[List[Dict[str, Any]], None]
        if isinstance(self.follow_up_questions, list):
            follow_up_questions = []
            for follow_up_questions_type_0_item_data in self.follow_up_questions:
                follow_up_questions_type_0_item = follow_up_questions_type_0_item_data.to_dict()
                follow_up_questions.append(follow_up_questions_type_0_item)

        else:
            follow_up_questions = self.follow_up_questions

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "cefr_level": cefr_level,
                "question_type": question_type,
                "question": question,
                "answers": answers,
                "follow_up_questions": follow_up_questions,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.placement_test_answer import PlacementTestAnswer
        from ..models.placement_test_follow_up_question import PlacementTestFollowUpQuestion

        d = src_dict.copy()
        cefr_level = d.pop("cefr_level")

        question_type = PlacementTestQuestionType(d.pop("question_type"))

        question = d.pop("question")

        answers = []
        _answers = d.pop("answers")
        for answers_item_data in _answers:
            answers_item = PlacementTestAnswer.from_dict(answers_item_data)

            answers.append(answers_item)

        def _parse_follow_up_questions(data: object) -> Union[List["PlacementTestFollowUpQuestion"], None]:
            if data is None:
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                follow_up_questions_type_0 = []
                _follow_up_questions_type_0 = data
                for follow_up_questions_type_0_item_data in _follow_up_questions_type_0:
                    follow_up_questions_type_0_item = PlacementTestFollowUpQuestion.from_dict(
                        follow_up_questions_type_0_item_data
                    )

                    follow_up_questions_type_0.append(follow_up_questions_type_0_item)

                return follow_up_questions_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List["PlacementTestFollowUpQuestion"], None], data)

        follow_up_questions = _parse_follow_up_questions(d.pop("follow_up_questions"))

        placement_test_question = cls(
            cefr_level=cefr_level,
            question_type=question_type,
            question=question,
            answers=answers,
            follow_up_questions=follow_up_questions,
        )

        placement_test_question.additional_properties = d
        return placement_test_question

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="GrammarErrorV2")


@_attrs_define
class GrammarErrorV2:
    """
    Attributes:
        type (str):
        start_position (int):
        end_position (int):
        replacement (str):
        explanation (str):
        sentence (str):
    """

    type: str
    start_position: int
    end_position: int
    replacement: str
    explanation: str
    sentence: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type

        start_position = self.start_position

        end_position = self.end_position

        replacement = self.replacement

        explanation = self.explanation

        sentence = self.sentence

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "start_position": start_position,
                "end_position": end_position,
                "replacement": replacement,
                "explanation": explanation,
                "sentence": sentence,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type")

        start_position = d.pop("start_position")

        end_position = d.pop("end_position")

        replacement = d.pop("replacement")

        explanation = d.pop("explanation")

        sentence = d.pop("sentence")

        grammar_error_v2 = cls(
            type=type,
            start_position=start_position,
            end_position=end_position,
            replacement=replacement,
            explanation=explanation,
            sentence=sentence,
        )

        grammar_error_v2.additional_properties = d
        return grammar_error_v2

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
